function dateUTCToParis(date) {
    return moment.tz(date, 'UTC').tz('Europe/Paris');
}

function KHistory(options = {}) {
    $.extend(this, KHistory.default_options, options);

    this.$container = $(this.container);

    this.$container.selectable({
        filter: 'div.opegroup, div.ope',
        selected: function (e, ui) {
            $(ui.selected).each(function () {
                if ($(this).hasClass('opegroup')) {
                    var opegroup = $(this).data('id');
                    $(this).siblings('.ope').filter(function () {
                        return $(this).data('opegroup') == opegroup
                    }).addClass('ui-selected');
                }
            });
        },
    });

    this.reset = function () {
        this.$container.html('');
    };

    this.addOpeGroup = function (opegroup) {
        var $day = this._getOrCreateDay(opegroup['at']);
        var $opegroup = this._opeGroupHtml(opegroup);

        $day.after($opegroup);

        var trigramme = opegroup['on_acc_trigramme'];
        var is_cof = opegroup['is_cof'];
        var type = opegroup['type']
        switch (type) {
            case 'opegroup':
                for (let ope of opegroup['opes']) {
                    var $ope = this._opeHtml(ope, is_cof, trigramme);
                    $ope.data('opegroup', opegroup['id']);
                    $opegroup.after($ope);
                }
                break;
            case 'transfergroup':
                for (let transfer of opegroup['opes']) {
                    var $transfer = this._transferHtml(transfer);
                    $transfer.data('transfergroup', opegroup['id']);
                    $opegroup.after($transfer);
                }
                break;
        }
    }

    this._opeHtml = function (ope, is_cof, trigramme) {
        var $ope_html = $(this.template_ope);
        var parsed_amount = parseFloat(ope['amount']);
        var amount = amountDisplay(parsed_amount, is_cof, trigramme);
        var infos1 = '', infos2 = '';

        if (ope['type'] == 'purchase') {
            infos1 = ope['article_nb'];
            infos2 = ope['article__name'] ? ope['article__name'] : 'Article supprimé';
        } else {
            infos1 = parsed_amount.toFixed(2) + '€';
            switch (ope['type']) {
                case 'initial':
                    infos2 = 'Initial';
                    break;
                case 'withdraw':
                    infos2 = 'Retrait';
                    break;
                case 'deposit':
                    infos2 = 'Charge';
                    break;
                case 'edit':
                    infos2 = 'Édition';
                    break;
            }
        }

        $ope_html
            .data('type', 'ope')
            .data('id', ope['id'])
            .find('.amount').text(amount).end()
            .find('.infos1').text(infos1).end()
            .find('.infos2').text(infos2).end();

        var addcost_for = ope['addcost_for__trigramme'];
        if (addcost_for) {
            var addcost_amount = parseFloat(ope['addcost_amount']);
            $ope_html.find('.addcost').text('(' + amountDisplay(addcost_amount, is_cof) + ' UKF pour ' + addcost_for + ')');
        }

        if (ope['canceled_at'])
            this.cancelOpe(ope, $ope_html);

        return $ope_html;
    }

    this._transferHtml = function (transfer) {
        var $transfer_html = $(this.template_transfer);
        var parsed_amount = parseFloat(transfer['amount']);
        var amount = parsed_amount.toFixed(2) + '€';

        $transfer_html
            .data('type', 'transfer')
            .data('id', transfer['id'])
            .find('.amount').text(amount).end()
            .find('.infos1').text(transfer['from_acc']).end()
            .find('.infos2').text(transfer['to_acc']).end();

        if (transfer['canceled_at'])
            this.cancelOpe(transfer, $transfer_html);

        return $transfer_html;
    }


    this.cancelOpe = function (ope, $ope = null) {
        if (!$ope)
            $ope = this.findOpe(ope['id'], ope["type"]);

        var cancel = 'Annulé';
        var canceled_at = dateUTCToParis(ope['canceled_at']);
        if (ope['canceled_by__trigramme'])
            cancel += ' par ' + ope['canceled_by__trigramme'];
        cancel += ' le ' + canceled_at.format('DD/MM/YY à HH:mm:ss');

        $ope.addClass('canceled').find('.canceled').text(cancel);
    }

    this._opeGroupHtml = function (opegroup) {
        var type = opegroup['type'];


        switch (type) {
            case 'opegroup':
                var $opegroup_html = $(this.template_opegroup);
                var trigramme = opegroup['on_acc__trigramme'];
                var amount = amountDisplay(
                    parseFloat(opegroup['amount']), opegroup['is_cof'], trigramme);
                break;
            case 'transfergroup':
                var $opegroup_html = $(this.template_transfergroup);
                $opegroup_html.find('.infos').text('Transferts').end()
                var trigramme = '';
                var amount = '';
                break;
        }


        var at = dateUTCToParis(opegroup['at']).format('HH:mm:ss');
        var comment = opegroup['comment'] || '';

        $opegroup_html
            .data('type', type)
            .data('id', opegroup['id'])
            .find('.time').text(at).end()
            .find('.amount').text(amount).end()
            .find('.comment').text(comment).end()
            .find('.trigramme').text(trigramme).end();

        if (!this.display_trigramme)
            $opegroup_html.find('.trigramme').remove();
        $opegroup_html.find('.info').remove();

        if (opegroup['valid_by__trigramme'])
            $opegroup_html.find('.valid_by').text('Par ' + opegroup['valid_by__trigramme']);

        return $opegroup_html;
    }

    this._getOrCreateDay = function (date) {
        var at = dateUTCToParis(date);
        var at_ser = at.format('YYYY-MM-DD');
        var $day = this.$container.find('.day').filter(function () {
            return $(this).data('date') == at_ser
        });
        if ($day.length == 1)
            return $day;
        var $day = $(this.template_day).prependTo(this.$container);
        return $day.data('date', at_ser).text(at.format('D MMMM YYYY'));
    }

    this.findOpeGroup = function (id) {
        return this.$container.find('.opegroup').filter(function () {
            return $(this).data('opegroup') == id
        });
    }

    this.findOpe = function (id, type = 'ope') {
        return this.$container.find('.ope').filter(function () {
            return ($(this).data('id') == id && $(this).data('type') == type)
        });
    }

    this.cancelOpeGroup = function (opegroup) {
        var $opegroup = this.findOpeGroup(opegroup['id']);
        var trigramme = $opegroup.find('.trigramme').text();
        var amount = amountDisplay(
            parseFloat(opegroup['amount']), opegroup['is_cof'], trigramme);
        $opegroup.find('.amount').text(amount);
    }

    this.fetch = function (fetch_options) {
        options = $.extend({}, this.fetch_options, fetch_options);
        var that = this;
        $.ajax({
            dataType: "json",
            url: django_urls["kfet.history.json"](),
            method: "POST",
            data: options,
        }).done(function (data) {
            for (let opegroup of data['opegroups']) {
                that.addOpeGroup(opegroup);
            }
        });
    }

    this.cancel = function (type, opes, password = "") {
        if (window.lock == 1)
            return false
        window.lock = 1;
        $.ajax({
            dataType: "json",
            url: django_urls[`kfet.${type}.cancel`](),
            method: "POST",
            data: opes,
            beforeSend: function ($xhr) {
                $xhr.setRequestHeader("X-CSRFToken", csrftoken);
                if (password != '')
                    $xhr.setRequestHeader("KFetPassword", password);
            },

        }).done(function (data) {
            window.lock = 0;
        }).fail(function ($xhr) {
            var data = $xhr.responseJSON;
            switch ($xhr.status) {
                case 403:
                    requestAuth(data, function (password) {
                        this.cancel(opes, password);
                    });
                    break;
                case 400:
                    displayErrors(getErrorsHtml(data));
                    break;
            }
            window.lock = 0;
        });
    }

    this.cancel_selected = function () {
        var opes_to_cancel = {
            "transfers": [],
            "operations": [],
        }
        this.$container.find('.ope.ui-selected').each(function () {
            if ($(this).data("transfergroup"))
                opes_to_cancel["transfers"].push($(this).data("id"));
            else
                opes_to_cancel["operations"].push($(this).data("id"));
        });
        if (opes_to_cancel["transfers"].length > 0 && opes_to_cancel["operations"].length > 0) {
            // Lancer 2 requêtes AJAX et gérer tous les cas d'erreurs possibles est trop complexe
            $.alert({
                title: 'Erreur',
                content: "Impossible de supprimer des transferts et des opérations en même temps !",
                backgroundDismiss: true,
                animation: 'top',
                closeAnimation: 'bottom',
                keyboardEnabled: true,
            });
        } else if (opes_to_cancel["transfers"].length > 0) {
            delete opes_to_cancel["operations"];
            this.cancel("transfers", opes_to_cancel);
        } else if (opes_to_cancel["operations"].length > 0) {
            delete opes_to_cancel["transfers"];
            this.cancel("operations", opes_to_cancel);
        }
    }
}

KHistory.default_options = {
    container: '#history',
    template_day: '<div class="day"></div>',
    template_opegroup: '<div class="opegroup"><span class="time"></span><span class="trigramme"></span><span class="amount"></span><span class="valid_by"></span><span class="comment"></span></div>',
    template_transfergroup: '<div class="opegroup"><span class="time"></span><span class="infos"></span><span class="valid_by"></span><span class="comment"></span></div>',
    template_ope: '<div class="ope"><span class="amount"></span><span class="infos1"></span><span class="infos2"></span><span class="addcost"></span><span class="canceled"></span></div>',
    template_transfer: '<div class="ope"><span class="amount"></span><span class="infos1"></span><span class="glyphicon glyphicon-arrow-right"></span><span class="infos2"></span><span class="canceled"></span></div>',
    display_trigramme: true,
}
